@@ -26,21 +26,19 @@ import warnings
 from enum import IntEnum
 import json
 from json import JSONEncoder
-from collections import OrderedDict, Mapping
 import operator
-import functools
 from ast import literal_eval
 
-try:
-    from ConfigParser import NoOptionError, NoSectionError
-except ImportError:
-    from configparser import NoOptionError, NoSectionError
+from configparser import NoOptionError, NoSectionError
 
+from luigi import date_interval
 from luigi import task_register
-from luigi import six
 from luigi import configuration
 from luigi.cmdline_parser import CmdlineParser
 
+from .freezing import recursively_freeze, FrozenOrderedDict
+
+
 _no_value = object()
 
 
@@ -89,7 +87,7 @@ class DuplicateParameterException(ParameterException):
     pass
 
 
-class Parameter(object):
+class Parameter:
     """
     Parameter whose value is a ``str``, and a base class for other parameter types.
 
@@ -216,9 +214,6 @@ class Parameter(object):
             found = getattr(cp_parser.known_args, dest, None)
             yield (self._parse_or_no_value(found), None)
         yield (self._get_value_from_config(task_name, param_name), None)
-        yield (self._get_value_from_config(task_name, param_name.replace('_', '-')),
-               'Configuration [{}] {} (with dashes) should be avoided. Please use underscores.'.format(
-                   task_name, param_name))
         if self._config_path:
             yield (self._get_value_from_config(self._config_path['section'], self._config_path['name']),
                    'The use of the configuration [{}] {} is deprecated. Please use [{}] {}'.format(
@@ -280,7 +275,7 @@ class Parameter(object):
     def _warn_on_wrong_param_type(self, param_name, param_value):
         if self.__class__ != Parameter:
             return
-        if not isinstance(param_value, six.string_types):
+        if not isinstance(param_value, str):
             warnings.warn('Parameter "{}" with value "{}" is not of type string.'.format(param_name, param_value))
 
     def normalize(self, x):
@@ -343,7 +338,7 @@ class OptionalParameter(Parameter):
     def _warn_on_wrong_param_type(self, param_name, param_value):
         if self.__class__ != OptionalParameter:
             return
-        if not isinstance(param_value, six.string_types) and param_value is not None:
+        if not isinstance(param_value, str) and param_value is not None:
             warnings.warn('OptionalParameter "{}" with value "{}" is not of type string or None.'.format(
                 param_name, param_value))
 
@@ -361,7 +356,8 @@ class _DateParameterBase(Parameter):
         self.interval = interval
         self.start = start if start is not None else _UNIX_EPOCH.date()
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def date_format(self):
         """
         Override me with a :py:meth:`~datetime.date.strftime` string.
@@ -436,7 +432,8 @@ class MonthParameter(DateParameter):
     (day of :py:class:`~datetime.date` is "rounded" to first of the month).
 
     A MonthParameter is a Date string formatted ``YYYY-MM``. For example, ``2013-07`` specifies
-    July of 2013.
+    July of 2013. Task objects constructed from code accept :py:class:`~datetime.date` (ignoring the day value) or
+    :py:class:`~luigi.date_interval.Month`.
     """
 
     date_format = '%Y-%m'
@@ -460,6 +457,9 @@ class MonthParameter(DateParameter):
         if value is None:
             return None
 
+        if isinstance(value, date_interval.Month):
+            value = value.date_a
+
         months_since_start = (value.year - self.start.year) * 12 + (value.month - self.start.month)
         months_since_start -= months_since_start % self.interval
 
@@ -471,7 +471,8 @@ class YearParameter(DateParameter):
     Parameter whose value is a :py:class:`~datetime.date`, specified to the year
     (day and month of :py:class:`~datetime.date` is "rounded" to first day of the year).
 
-    A YearParameter is a Date string formatted ``YYYY``.
+    A YearParameter is a Date string formatted ``YYYY``. Task objects constructed from code accept
+    :py:class:`~datetime.date` (ignoring the month and day values) or :py:class:`~luigi.date_interval.Year`.
     """
 
     date_format = '%Y'
@@ -483,6 +484,9 @@ class YearParameter(DateParameter):
         if value is None:
             return None
 
+        if isinstance(value, date_interval.Year):
+            value = value.date_a
+
         delta = (value.year - self.start.year) % self.interval
         return datetime.date(year=value.year - delta, month=1, day=1)
 
@@ -497,14 +501,16 @@ class _DatetimeParameterBase(Parameter):
         self.interval = interval
         self.start = start if start is not None else _UNIX_EPOCH
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def date_format(self):
         """
         Override me with a :py:meth:`~datetime.date.strftime` string.
         """
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def _timedelta(self):
         """
         How to move one interval of this type forward (i.e. not counting self.interval).
@@ -750,7 +756,7 @@ class TimeDeltaParameter(Parameter):
         if re_match and any(re_match.groups()):
             kwargs = {}
             has_val = False
-            for k, v in six.iteritems(re_match.groupdict(default="0")):
+            for k, v in re_match.groupdict(default="0").items():
                 val = int(v)
                 if val > -1:
                     has_val = True
@@ -883,57 +889,61 @@ class EnumParameter(Parameter):
         return e.name
 
 
-class _FrozenOrderedDict(Mapping):
+class EnumListParameter(Parameter):
     """
-    It is an immutable wrapper around ordered dictionaries that implements the complete :py:class:`collections.Mapping`
-    interface. It can be used as a drop-in replacement for dictionaries where immutability and ordering are desired.
+    A parameter whose value is a comma-separated list of :class:`~enum.Enum`. Values should come from the same enum.
+
+    Values are taken to be a list, i.e. order is preserved, duplicates may occur, and empty list is possible.
+
+    In the task definition, use
+
+    .. code-block:: python
+
+        class Model(enum.Enum):
+          Honda = 1
+          Volvo = 2
+
+        class MyTask(luigi.Task):
+          my_param = luigi.EnumListParameter(enum=Model)
+
+    At the command line, use,
+
+    .. code-block:: console
+
+        $ luigi --module my_tasks MyTask --my-param Honda,Volvo
+
     """
 
+    _sep = ','
+
     def __init__(self, *args, **kwargs):
-        self.__dict = OrderedDict(*args, **kwargs)
-        self.__hash = None
+        if 'enum' not in kwargs:
+            raise ParameterException('An enum class must be specified.')
+        self._enum = kwargs.pop('enum')
+        super(EnumListParameter, self).__init__(*args, **kwargs)
 
-    def __getitem__(self, key):
-        return self.__dict[key]
+    def parse(self, s):
+        values = [] if s == '' else s.split(self._sep)
 
-    def __iter__(self):
-        return iter(self.__dict)
+        for i, v in enumerate(values):
+            try:
+                values[i] = self._enum[v]
+            except KeyError:
+                raise ValueError('Invalid enum value "{}" index {} - could not be parsed'.format(v, i))
 
-    def __len__(self):
-        return len(self.__dict)
+        return tuple(values)
 
-    def __repr__(self):
-        return '<FrozenOrderedDict %s>' % repr(self.__dict)
-
-    def __hash__(self):
-        if self.__hash is None:
-            hashes = map(hash, self.items())
-            self.__hash = functools.reduce(operator.xor, hashes, 0)
-
-        return self.__hash
-
-    def get_wrapped(self):
-        return self.__dict
-
-
-def _recursively_freeze(value):
-    """
-    Recursively walks ``Mapping``s and ``list``s and converts them to ``_FrozenOrderedDict`` and ``tuples``, respectively.
-    """
-    if isinstance(value, Mapping):
-        return _FrozenOrderedDict(((k, _recursively_freeze(v)) for k, v in value.items()))
-    elif isinstance(value, list) or isinstance(value, tuple):
-        return tuple(_recursively_freeze(v) for v in value)
-    return value
+    def serialize(self, enum_values):
+        return self._sep.join([e.name for e in enum_values])
 
 
 class _DictParamEncoder(JSONEncoder):
     """
-    JSON encoder for :py:class:`~DictParameter`, which makes :py:class:`~_FrozenOrderedDict` JSON serializable.
+    JSON encoder for :py:class:`~DictParameter`, which makes :py:class:`~FrozenOrderedDict` JSON serializable.
     """
 
     def default(self, obj):
-        if isinstance(obj, _FrozenOrderedDict):
+        if isinstance(obj, FrozenOrderedDict):
             return obj.get_wrapped()
         return json.JSONEncoder.default(self, obj)
 
@@ -973,11 +983,11 @@ class DictParameter(Parameter):
 
     def normalize(self, value):
         """
-        Ensure that dictionary parameter is converted to a _FrozenOrderedDict so it can be hashed.
+        Ensure that dictionary parameter is converted to a FrozenOrderedDict so it can be hashed.
         """
-        return _recursively_freeze(value)
+        return recursively_freeze(value)
 
-    def parse(self, s):
+    def parse(self, source):
         """
         Parses an immutable and ordered ``dict`` from a JSON string using standard JSON library.
 
@@ -988,7 +998,10 @@ class DictParameter(Parameter):
 
         :param s: String to be parse
         """
-        return json.loads(s, object_pairs_hook=_FrozenOrderedDict)
+        # TOML based config convert params to python types itself.
+        if not isinstance(source, str):
+            return source
+        return json.loads(source, object_pairs_hook=FrozenOrderedDict)
 
     def serialize(self, x):
         return json.dumps(x, cls=_DictParamEncoder)
@@ -1032,7 +1045,7 @@ class ListParameter(Parameter):
         :param str x: the value to parse.
         :return: the normalized (hashable/immutable) value.
         """
-        return _recursively_freeze(x)
+        return recursively_freeze(x)
 
     def parse(self, x):
         """
@@ -1041,7 +1054,7 @@ class ListParameter(Parameter):
         :param str x: the value to parse.
         :return: the parsed value.
         """
-        return list(json.loads(x, object_pairs_hook=_FrozenOrderedDict))
+        return list(json.loads(x, object_pairs_hook=FrozenOrderedDict))
 
     def serialize(self, x):
         """
@@ -1104,9 +1117,9 @@ class TupleParameter(ListParameter):
         # ast.literal_eval(t_str) == t
         try:
             # loop required to parse tuple of tuples
-            return tuple(tuple(x) for x in json.loads(x, object_pairs_hook=_FrozenOrderedDict))
-        except ValueError:
-            return literal_eval(x)  # if this causes an error, let that error be raised.
+            return tuple(tuple(x) for x in json.loads(x, object_pairs_hook=FrozenOrderedDict))
+        except (ValueError, TypeError):
+            return tuple(literal_eval(x))  # if this causes an error, let that error be raised.
 
 
 class NumericalParameter(Parameter):
